#include "creargrafodlg.h"

#include <qlayout.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qmessagebox.h>

//! Crea un cuadro de dilogo Crear grafo

/*!
    Crea el cuadro de dilogo.

    \param parent Widget padre.
    \param name Nombre del widget.
    \param modal Indica si el dilogo ser o no modal.
    \param f Opciones de creacin del widget.
 */
CrearGrafoDlg::CrearGrafoDlg ( QWidget *parent = 0, const char *name = 0,
    bool modal = FALSE, WFlags f = 0 ):QDialog( parent, name, modal, f )
{
    // Creamos una caja vertical
    QVBoxLayout *vb = new QVBoxLayout( this, 8 );

    // Ahora una horizontal
    QHBoxLayout *hb = new QHBoxLayout( vb );

    // Ponemos una etiqueta en la caja horizontal
    hb->addWidget( new QLabel( "Nombre", this ) );

    // Y una entrada de texto
    _nombre = new QLineEdit( this );
    _nombre->setText( "Sin Nombre" );
    hb->addWidget( _nombre );

    // Creamos otra caja horizontal
    hb = new QHBoxLayout( vb );

    // Ponemos una etiqueta en la caja horizontal
    hb->addWidget( new QLabel( "Tipo de grafo", this ) );

    // Y un combo en la misma
    _tipoGrafo = new QComboBox( false, this );
    hb->addWidget( _tipoGrafo );
    _tipoGrafo->insertItem( "Dirigido" );
    _tipoGrafo->insertItem("No dirigido");

    // Creamos otra caja horizontal
    hb = new QHBoxLayout( vb );

    // Ponemos una etiqueta en la caja
    hb->addWidget( new QLabel( "Nmero de nodos", this ) );

    // Y una entrada de texto
    _numNodos = new QLineEdit( this );
    _numNodos->setText( "5" );
    hb->addWidget( _numNodos );

    // Creamos otra caja horizontal
    hb = new QHBoxLayout( vb );

    // El botn aceptar
    QPushButton *aceptar=new QPushButton( "Aceptar", this );
    hb->addWidget( aceptar );
    aceptar->setDefault( true );
    connect( aceptar, SIGNAL( clicked() ), this, SLOT( accept() ) );

    // El botn cancelar
    QPushButton *cancelar=new QPushButton( "Cancelar", this );
    hb->addWidget( cancelar );
    connect( cancelar, SIGNAL( clicked() ), this, SLOT( reject() ) );

}

//! Destruye el cuadro de dilogo

/*!
    Destruye el cuadro de dilogo.
 */
CrearGrafoDlg::~CrearGrafoDlg( void )
{
    // Los widgets hijos se liberan solos
}

//! El usuario acepta los datos

/*!
    Cdigo a ejecutar cuando el ususario acepta los datos.
 */
void CrearGrafoDlg::accept( void )
{
    int resultado;
    bool ok;

    resultado = _numNodos->text().toInt( &ok );

    if ( !ok || resultado<=0 ) {
        QMessageBox::critical( 0, "Error", "Nmero de nodos no vlido" );
    } else {
        done( Accepted );
    }
}

//! Nombre del grafo

/*!
    Nombre que el usuario da al nuevo grafo.

    \returns Nombre del grafo a crear.
 */
QString CrearGrafoDlg::nombre( void )
{
    return _nombre->text();
}

//! Nmero de nodos del grafo

/*!
    Nmero de nodos del nuevo grafo.

    \returns Nmero de nodos del grafo a crear.
 */
int CrearGrafoDlg::numNodos( void )
{
    return _numNodos->text().toInt();
}

//! Tipo del grafo

/*!
    Tipo del nuevo grafo.

    \returns Tipo del grafo a crear.
 */
CrearGrafoDlg::TipoGrafo CrearGrafoDlg::tipoGrafo( void )
{
    if ( _tipoGrafo->currentItem() == 0 ) {
        return Dirigido;
    } else {
        return NoDirigido;
    }
}
