#include "documento.h"
#include "vistagrafo.h"
#include "grafo.h"
#include "introducirpesosdlg.h"

#include <assert.h>

//! Crea un documento

/*!
    Crea un nuevo documento.

    \param g Grafo a gestionar.
    \param parent Widget padre.
    \param name Nombre del widget.
    \param f Opciones de creacin del widget.
 */
Documento::Documento(  Grafo *g, QWidget * parent = 0, const char * name = 0,
            WFlags f = WType_TopLevel ):QMainWindow( parent, name, f )
{
    assert( g!=NULL );
    grafo( g );
    _vista = new VistaGrafo( g, this );
    setFocusProxy( _vista );
    setCentralWidget( _vista );
    connect( this, SIGNAL( grafoModificado() ),
        _vista, SLOT( grafoModificado() ) );

}

//! Destruye el documento

/*!
    Destruye el documento.
 */
Documento::~Documento( void )
{
    delete _grafo;
}

//! Grafo que gestiona el documento

/*!
    Devulve el grafo que gestiona este documento.

    \returns Grafo que gestiona.
 */
Grafo *Documento::grafo() const
{
    return _grafo;
}

//! Asigna el grafo

/*!
    Asigna el grafo a gestionar.
 */
void Documento::grafo( Grafo *g )
{
    _grafo = g;
}

//! Modifica los pesos del grafo

/*!
    Muestra el cuadro de dialogo para editar los pesos del grafo.
 */
void Documento::modificarPesos( void )
{
    IntroducirPesosDlg dlgPesos( grafo(), this, 0, true );
    dlgPesos.setCaption( "Introduzca los pesos del grafo" );
    if ( dlgPesos.exec() == IntroducirPesosDlg::Accepted ) {
        emit( grafoModificado() );
    }
}
