#ifndef __GRAFO_H__
#define __GRAFO_H__

#include <qstring.h> // esto para que pille la cadena
//! Clase grafo

/*!
    Esta es la clase que representa a un grafo, sea este dirigido
    o no dirigido.
 */
class Grafo {
    public:
        Grafo( int n );
        virtual ~Grafo();
        int numNodos( void ) const;
// funciones de practica.

	void potencia(int n);
        int ** floyd (int origen);
	void caminofloyd(int **P, int i, int j, QString &cadena); 

        //! Tipo del peso

        /*!
            Tipo de dato del peso de los arcos.
         */
        typedef int TipoPeso;


        TipoPeso peso( int i, int j ) const;

        //! Asigna un peso a un arco
        /*!
            Asigna el peso del arco que conecta los nodos
            <tt>i</tt> y <tt>j</tt>.

            \param i Origen del arco.
            \param j Destino del arco.
            \param valor Peso del arco.
         */
        virtual void peso( int i, int j, TipoPeso valor )=0;

        //! Valor del peso de un arco no existente

        /*!
            Los arcos con este peso no existen.
         */
        static const TipoPeso infinito=32000;

        void podarMayores( TipoPeso umbral );

        // Crea una copia del grafo

        /*!
            Crea un grafo que es copia de este.

            \returns Nuevo grafo.
         */
        virtual Grafo *copia( void )=0;

    protected:
        void inicializar( int n );
        void liberar( void );
        void copiaPesos( Grafo *otro );

        //! Nmero de nodos del grafo.

        /*!
            Nmero de nodos en el grafo. Siempre es mayor que cero.
         */
        int _numNodos;

        //! Matriz de conexiones del grafo.

        /*!
            La matriz de conexiones del grafo.
         */
        TipoPeso **_matriz;
};

#endif
