#include "grafodirigido.h"

#include <assert.h>

//! Construye un grafo dirigido

/*!
    Construye un grafo dirigido de <tt>n</tt>
    nodos.

    \param n Nmero de nodos.
 */
GrafoDirigido::GrafoDirigido( int n ):Grafo( n )
{

}

//! Destruye un grafo dirigido

/*!
    Destruye un grafo dirigido.
 */
GrafoDirigido::~GrafoDirigido()
{

}

//! Asigna un peso a un arco

/*!
    Asigna el peso del arco que conecta los nodos
    <tt>i</tt> y <tt>j</tt>.

    Requiere i>=0 y i<numNodos()
    y j>=0 y j<numNodos() y valor <= infinito.

    Asegura valor == peso( i, j ).

    \param i Origen del arco.
    \param j Destino del arco.
    \param valor Peso del arco.
 */
void GrafoDirigido::peso( int i, int j, TipoPeso valor )
{
    assert( i>=0 && i<numNodos() );
    assert( j>=0 && j<numNodos() );
    assert( valor <= infinito );
    _matriz[i][j] = valor;
}

// Crea una copia del grafo

/*!
    Crea un grafo que es copia de este.

    \returns Nuevo grafo.
*/
Grafo *GrafoDirigido::copia( void )
{
    GrafoDirigido *nuevo=new GrafoDirigido( numNodos() );
    nuevo->copiaPesos( this );
    return nuevo;
}
