#include "practica.h"
#include "documento.h"
#include "creargrafodlg.h"
#include "grafodirigido.h"
#include "grafonodirigido.h"
#include "introducirpesosdlg.h"
#include "vistagrafo.h"

#include <stdio.h>
#include <qworkspace.h>
#include <qlistbox.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qinputdialog.h>
#include <qmessagebox.h>
#include <qpixmap.h>
#include <qvbox.h>


// Declara la variable UCOsmall_xpm como pixmap
#include "ucosmall.xpm"

//! Construye un objeto de la clase Practica

/*!
    Comstruye un objeto de la clase Practica.

    \param parent Widget padre.
    \param name Nombre del widget.
    \param f Opciones del widget.
 */
Practica::Practica ( QWidget * parent = 0, const char * name = 0,
                     WFlags f = WType_TopLevel ):QMainWindow( parent, name, f )
{
    // Creamos el interfaz de usuario
    crearInterfaz();
}

//! Destruye un objeto de la clase Practica.

/*!
    Destruye un objeto de la clase Practica.
 */
Practica::~Practica()
{
    // Los widgets hijo se liberan slos
}

//! Crea el interfaz de usuario

/*!
    Este mtodo inicializa el interfaz de usuario.
 */
void Practica::crearInterfaz( void )
{
    // Ponemos el caption y el icono de la ventana
	setCaption( "Prctica de grafos" );
	setIcon( QPixmap( UCOsmall_xpm ) );

    // Creamos el men archivo
	QPopupMenu *menuArchivo = new QPopupMenu( this );
    menuArchivo->insertItem( "Nuevo", this, SLOT(nuevo()),CTRL+Key_N );
    menuArchivo->insertSeparator();
	menuArchivo->insertItem( "Salir", this, SLOT(close()),CTRL+Key_Q );
	menuBar()->insertItem( "Archivo", menuArchivo );

    // Creamos el men editar
	QPopupMenu *menuEditar = new QPopupMenu( this );
    menuEditar->insertItem( "Seleccionar todo", this,
        SLOT(seleccionarTodo()) );
    menuEditar->insertItem( "Deseleccionar todo", this,
        SLOT(deseleccionarTodo()) );
	menuBar()->insertItem( "Editar", menuEditar );

    // El men grafo
	QPopupMenu *menuGrafo = new QPopupMenu( this );
    menuGrafo->insertItem( "Modificar pesos", this, SLOT( modificarPesos()),CTRL+Key_M );
    menuGrafo->insertSeparator();
    menuGrafo->insertItem( "Podar grafo", this, SLOT( podarGrafo() ),CTRL+Key_P );
    menuGrafo->insertItem( "Potencia", this, SLOT( potencia_n() ),CTRL+Key_E );
    menuGrafo->insertItem( "Caminos Mnimos", this, SLOT( CaminosMinimos()),CTRL+Key_R );
    menuGrafo->insertItem( "Red Minima Prim", this, SLOT( RedMinimaPrim()),CTRL+Key_R );
    menuGrafo->insertItem( "Red Maxima Prim", this, SLOT( RedMaximaPrim()),CTRL+Key_R );
	
menuBar()->insertItem( "Grafo", menuGrafo );

    // Ahora el men nodo
	QPopupMenu *menuNodo = new QPopupMenu( this );
	menuBar()->insertItem( "Nodo", menuNodo );

    // El men de recorridos
    QPopupMenu *menuRecorrido= new QPopupMenu( this );
    menuRecorrido->insertItem("Clasificacin topolgica", this,
        SLOT( clasificacionTopologica() ) );
	menuBar()->insertItem( "Recorrido", menuRecorrido );

    // El menu ventana (lo tenemos que hacer al vuelo)
    menuVentana = new QPopupMenu( this );
    connect( menuVentana, SIGNAL( aboutToShow() ),
             this, SLOT( mostrarMenuVentana() ) );
    menuBar()->insertItem( "Ventana", menuVentana );

    // El men de ayuda
	QPopupMenu *menuAyuda = new QPopupMenu( this );
	menuAyuda->insertItem( "Acerca de...", this, SLOT(about()) );
	menuBar()->insertItem( "Ayuda", menuAyuda );

    QVBox* vb = new QVBox( this );
    vb->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );
    ws = new QWorkspace( vb );
    setCentralWidget( vb );
}

//! Crea un nuevo grafo.

/*!
    Crea un nuevo documento grafo.
 */
void Practica::nuevo( void )
{
    CrearGrafoDlg dlg( this, 0, true );
    dlg.setCaption( "Nuevo grafo" );

    if ( dlg.exec() == CrearGrafoDlg::Accepted ) {
        // Creamos el grafo
        Grafo *g;
        if ( dlg.tipoGrafo() == CrearGrafoDlg::Dirigido ) {
            g = new GrafoDirigido( dlg.numNodos() );
        } else {
            g = new GrafoNoDirigido( dlg.numNodos() );
        }
        IntroducirPesosDlg dlgPesos( g, this, 0, true );
        dlgPesos.setCaption( "Introduzca los pesos del grafo" );
        dlgPesos.exec();

        Documento *d=new Documento( g, ws, 0, WDestructiveClose );
        d->setCaption( dlg.nombre() );
        d->setIcon( QPixmap( UCOsmall_xpm ) );
        d->show();
    }
}

//! Selecciona todos los nodos del grafo actual

/*!
    Selecciona todos los nodos del grafo actual. Observad
    como el mensaje se redirige a la vista.
 */
void Practica::seleccionarTodo( void )
{
    Documento *d=dynamic_cast<Documento *>(ws->activeWindow());
    if (d) {
        VistaGrafo *v=d->vista();
        v->seleccionarTodo( true );
    }
}

//! Deselecciona todos los nodos del grafo actual

/*!
    Deselecciona todos los nodos del grafo actual. Observad
    como el mensaje se redirige a la vista.
 */
void Practica::deseleccionarTodo( void )
{
    Documento *d=dynamic_cast<Documento *>(ws->activeWindow());
    if (d) {
        VistaGrafo *v=d->vista();
        v->seleccionarTodo( false );
    }
}

void Practica::clasificacionTopologica( void )
{
    // Por hacer
}

//! Poda el grafo

/*!
    Poda los arcos del grafo con un peso mayor que un umbral.
 */
void Practica::podarGrafo( void )
{
    bool aceptar = true;
    int umbral = QInputDialog::getInteger(
        "Podar arcos con peso mayor que un umbral",
        "Introduzca un valor umbral",
        1, 1, 1000, 1, &aceptar, this );

    if ( aceptar ) {
        Documento *d=dynamic_cast<Documento *>(ws->activeWindow());
        if (d) {
            Grafo *g=d->grafo();
            Grafo *nuevo=g->copia();
            nuevo->podarMayores( umbral );
            Documento *nd=new Documento( nuevo, ws, 0, WDestructiveClose );
            nd->setCaption( d->caption()+" podado" );
            nd->setIcon( QPixmap( UCOsmall_xpm ) );
            nd->show();
        }
    }
}
//! Potencia n-esima de un grafo(Orden de un grafo)

/*!
    Eleva la matriz de conexiones del grafo a la potencia n.
 */
void Practica::potencia_n( void )
{
    bool aceptar = true;
    int numero = QInputDialog::getInteger(
        "Potencia n-esima",
        "Introduzca a que grado quiere elevar el grafo",
        1, 1, 1000, 1, &aceptar, this );
    
    if ( aceptar ) 
    {
        Documento *d=dynamic_cast<Documento *>(ws->activeWindow());
        if (d) 
	    {
            char cadena[15];
	    sprintf(cadena," de orden %d",numero);
	    // aqui ponemos el numero de veces que elevamos la matriz de conexion a titulo informativo
	    Grafo *g=d->grafo();
            Grafo *nuevo=g->copia();
            nuevo->potencia( numero );
            Documento *nd=new Documento( nuevo, ws, 0, WDestructiveClose );
            nd->setCaption( d->caption()+cadena ); // le asignamos el nombre al documento
            nd->setIcon( QPixmap( UCOsmall_xpm ) );
            nd->show();
    	    }
    }
}

void Practica::CaminosMinimos(void)
{
    bool aceptar = true; 
    // i va a ser el nodo origen
    int i = QInputDialog::getInteger(
        "Problema de las plazas",
        "Introduzca la plaza de inicio",
        0, 0, 1000, 1, &aceptar, this );

   QString cad;
// creamos una lista para ir metiendo los caminos.
    QListBox *lista = new QListBox( NULL, NULL, Qt::WDestructiveClose );

    lista->setCaption("Caminos minimos desde la plaza "+QString::number(i));
    int **P;

    if ( aceptar ) {
        Documento *d=dynamic_cast<Documento *>(ws->activeWindow());
        if (d) {
            Grafo *g=d->grafo();
	    Grafo *nuevo=g->copia();
	    P=nuevo->floyd(i); // P va a ser la matriz de nudos intermedios.
	    Documento *nd=new Documento(nuevo, ws, 0, WDestructiveClose );
            nd->setCaption( d->caption()+"Caminos mas cortos desde la plaza "+QString::number(i));
            nd->setIcon( QPixmap( UCOsmall_xpm ) );
            nd->show();

// Calculamos los caminos que estan guardados en P
	  for(int j=0;j<nuevo->numNodos();j++)
	   {
                   if(i!=j && nuevo->peso(i,j)!=Grafo::infinito)
                   { cad=" ";
                    nuevo->caminofloyd(P,i,j,cad); //
		    new QListBoxText( lista,QString::number(i)+"-"+QString::number(j)+" :"+cad);
                    }

	   }
	   lista->show(); // monstramos la lista.
        }
    }


}

//Arbolminimal
void Practica::RedMinimaPrim(void)
{

bool aceptar=true;

        if(aceptar)
	{
          Documento *d=dynamic_cast<Documento *>(ws->activeWindow());
          if (d) {
                Grafo *g=d->grafo();
                Grafo *nuevo=g->copia();
                GrafoNoDirigido *aux=dynamic_cast<GrafoNoDirigido *>(nuevo->copia());                
		if(aux){
                        aux->PrimMinimo();
                        nuevo=aux->copia();
                        Documento *nd=new Documento(nuevo, ws, 0, WDestructiveClose );
                        nd->setCaption( d->caption()+"Problema 3 Minima ");
                        nd->setIcon( QPixmap( UCOsmall_xpm ) );
                        nd->show();
                 }
                 else
                 {
                        QMessageBox::critical(this,  "No es posible","Esto es solo valido para grafos NO DIRIGIDOS");
                 }
        }
    }

}

//Arbolmaximal
void Practica::RedMaximaPrim(void)
{

bool aceptar=true;

        if(aceptar)
	{
          Documento *d=dynamic_cast<Documento *>(ws->activeWindow());
          if (d) {
                Grafo *g=d->grafo();
                Grafo *nuevo=g->copia();
                GrafoNoDirigido *aux=dynamic_cast<GrafoNoDirigido *>(nuevo->copia());                
		if(aux){
                        aux->PrimMaximo();
                        nuevo=aux->copia();
                        Documento *nd=new Documento(nuevo, ws, 0, WDestructiveClose );
                        nd->setCaption( d->caption()+"Problema 3 Maxima ");
                        nd->setIcon( QPixmap( UCOsmall_xpm ) );
                        nd->show();
                 }
                 else
                 {
                        QMessageBox::critical(this,  "No es posible","Esto es solo valido para grafos NO DIRIGIDOS");
                 }
        }
    }

}



//! Modifica los pesos del grafo actual.

/*!
    Modifica los pesos del grafo actual. Observad
    como el mensaje se redirige al documento.
 */
void Practica::modificarPesos( void )
{
    Documento *d=dynamic_cast<Documento *>(ws->activeWindow());
    if (d) {
        d->modificarPesos();
    }
}

//! Muestra el menu ventana.

/*!
    Muestra el menu ventana. Hay que construirlo al vuelo
    para mostrar la lista de ventanas.
 */
void Practica::mostrarMenuVentana( void )
{
    menuVentana->clear();
    int cascadeId = menuVentana->insertItem("Cascada", ws, SLOT(cascade() ) );
    int tileId = menuVentana->insertItem("Mosaico", ws, SLOT(tile() ) );
    if ( ws->windowList().isEmpty() ) {
        menuVentana->setItemEnabled( cascadeId, FALSE );
        menuVentana->setItemEnabled( tileId, FALSE );
    }
    menuVentana->insertSeparator();
    QWidgetList ventanas = ws->windowList();
    for ( int i = 0; i < int(ventanas.count()); ++i ) {
        int id = menuVentana->insertItem(ventanas.at(i)->caption(),
            this, SLOT( mostrarVentana( int ) ) );
        menuVentana->setItemParameter( id, i );
        menuVentana->setItemChecked( id, ws->activeWindow() == ventanas.at(i) );
    }
}

//! Muestra una ventana.

/*!
    Muestra la ventana identificada por <tt>id</tt>.

    \param id Nmero de la ventana a mostrar.
 */
void Practica::mostrarVentana( int id )
{
    QWidget* w = ws->windowList().at( id );
    if ( w )
        w->showNormal();
    w->setFocus();
}

//! Muestra el cuadro de dilogo 'Acerca de...'

/*!
    Muestra el cuadro de dilogo 'Acerca de...'
 */
void Practica::about( void )
{
	QMessageBox::about(this, tr( "Acerca de Prctica de grafos" ),
	    tr( "Esqueleto para la <b>prctica de grafos</b><br>"
	    "Estructuras de datos y de la Informacin<br>"
	    "Curso 2001/2002<br>"
	    "Creado por Manuel Prieto Villegas cc1prvim@uco.es<br>"
	    "Grupo de investigacin Aplicaciones de la Visin Artificial<br>"
	    "Departamento de Informtica y Anlisis Numrico<br>"
	    "Universidad de Crdoba<br>" ) );
}
